.tree {
  .label {
    font-size: 90%;
    font-weight: normal;
    padding: 3px 7px !important;
  }

  .candidate .term {
    font-style: italic;
  }

  .archived .term {
    text-decoration: line-through;
  }

  i.extract-checker {
    cursor: pointer;
  }

  i.extension {
    cursor: pointer;
    transform: scale(1.5);
  }
}

.terms-widget {
  header.active {
    color: var(--body-bg);
    background-color: var(--primary-light) !important;
  }
}
